// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/

var ClientRequest = require('../util/apim-request');
var Util = require('../util/util');
var _ = require('lodash');
var debug = require('debug')('apiconnect-cli-util:organizations');
var g = require('strong-globalize')();
var request = require('request-promise');

/**
 * List provider orgs associated to this username/password
 *
 * @param username {String}
 * @param password {String}
 * @param host {String} API Management hostname:port
 * @param cb {Function(err, rsp, body)} callback function
 */
function listOrgs(host, cb) {
  var opts = Util.requestDefaultOptions(host);
  opts.url += '/v1/me/orgs?orgType=apiProvider';
  ClientRequest.get(opts, function(err, rsp, body) {
    cb(err, rsp, body);
  });
}

/**
 * List provider orgs associated to this username/password
 *
 * @param username {String}
 * @param password {String}
 * @param host {String} API Management hostname:port
 */
function listOrgsAsync(host) {
  var opts = Util.requestDefaultOptions(host);
  opts.url += '/v1/me/orgs?orgType=apiProvider';
  opts.method = 'get';
  opts.json = true;
  return request(opts).then(function(orgs) {
    return _.map(orgs, function(org) {
      if (org.externalNativeOrgId) {
        org.bluemix = {
          inviteUri: org.externalInviteOverride,
          organization: org.externalNativeOrgId,
          space: org.externalSpaceId,
        };
      }
      return org;
    });
  });
}

/**
 * Find a provider org associated to this username/password
 *
 * @param username {String}
 * @param password {String}
 * @param host {String} API Management hostname:port
 * @param orgName {String}
 */
function findOrgByNameAsync(host, orgName) {
  return listOrgsAsync(host).then(function(orgs) {
    var providerOrg = _.filter(orgs, { name: orgName });
    if (providerOrg.length !== 1) {
      throw new Error(g.f('No such provider organization: %s', orgName));
    }
    return providerOrg[0];
  });
}

/**
 * List orgs associated to the logged in user
 *
 * @param cb {Function (err, rsp, body)}
 */
function list(cb) {
  var opts = Util.requestDefaultOptions();
  opts.url += '/v1/me/orgs?orgType=apiProvider';
  ClientRequest.get(opts, function(err, rsp, body) {
    debug(body);
    cb(err, rsp, body);
  });
};

module.exports = {
  listOrgs: listOrgs,
  listOrgsAsync: listOrgsAsync,
  list: list,
  findOrgByNameAsync: findOrgByNameAsync,
};
