// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/

var _ = require('lodash');
var logger = require('apiconnect-cli-logger');

/**
 * Default constructor - extends the {Entity} properties
 *
 * @param entity  {Entity}
 * @constructor
 */
function Product(entity) {
  _.extend(this, entity);
}

Product.prototype.dump = function() {
  logger.info(this.getContent());
};
/**
 * Return the default product visibility
 *
 * @return {object}
 */
Product.prototype.getVisibility = function() {
  var product = this.getContent();
  return product.visibility ||
    {
      view: {
        enabled: true,
        type: 'public',
      },
      subscribed: {
        enabled: true,
      },
    };
};


/**
 * Return the environments properties defined within a product
 *
 * @return {[object]} an array of environments or empty array if none is found
 */
Product.prototype.getEnvironments = function() {
  var product = this.getContent();
  return product.environments || [];
};
/**
 * Find named environment <tt>envName</tt>
 *
 * @param envName
 * @param {object} return environment object or undefined
 */
Product.prototype.findEnvironment = function(/**String*/envName) {
  return this.getEnvironments()[envName];
};
/**
 * Return an array of string that contains the name of the plans associated to this product.
 * @return {Array{String}}
 */
Product.prototype.getPlanNames = function() {
  var product = this.getContent();
  return _.keys(product.plans);
};
/**
 * Return the plan named <tt>planName</tt> as it is defined within a product file.
 *
 * @param planName
 * @param resolve - if true resolve the plan and add the global apis if is not defined
 * @return {object} plan
 */
Product.prototype.findPlan = function(planName, resolve) {
  var product = this.getContent();
  var plan = product.plans[planName];
  if (resolve && plan && !plan.apis) {
    plan.apis = this.getApis();
  }
  return plan;
};
/**
 * Return the apis that are part of the products (apis section)
 *
 * @return{[object]} the apis defined within a product. {'Routes': {$ref: ./file}, 'ClimbOn': {$ref: ./file2}}}
 */
Product.prototype.listAllApis = function() {
  return this.getContent().apis;
};
/**
 * Return apis defined at the product level - the plans that have no apis defined within will make use of
 * the apis defined at this level.
 *
 * @param {Array(object)}  an array of object that represents the api defined at the product level;
 *                         for ex: [ {$ref: file:./api_swagger.yml}, {$ref: 'petstore:1.0.0'}]
 */
Product.prototype.getApis = function() {
  var product = this.getContent();
  return product.apis || [];
};

module.exports = Product;
