// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/

'use strict';

var Promise = require('bluebird');
var _ = require('lodash');
var apimClient = require('apiconnect-apim-client');
var g = require('strong-globalize')();

function RemoteApps(options) {
  this.__options = options;
}

RemoteApps.prototype.init = function() {
  return Promise.resolve();
};

RemoteApps.resolve = function(options) {
  options.serviceFromConfig = !options.service;
  options.name = options.organization;
  var providerOrgPromise = apimClient.orgs.get(options);
  return providerOrgPromise.then(function(org) {
    if (!options.app) {
      return Promise.resolve();
    }
    return apimClient.apps.get(_.defaults({
      organization: org.name,
      getEncrypted: true,
    }, options));
  }).then(function(providerApp) {
    var providerOrg = providerOrgPromise.value();
    options.providerApp = providerApp;
    options.providerOrg = providerOrg;
    if (options.providerApp.bluemix) {
      // Defered require due to cycles
      var Bluemix = require('../bluemix');
      return new Bluemix(options);
    } else if (options.providerApp.liberty) {
      // Defered require due to cycles
      var LibertySupport = require('../collectives');
      return new LibertySupport(options);
    } else {
      throw new Error(g.f('Unsupported application runtime'));
    }
  }).then(function(remote) {
    return remote.init().return(remote);
  });
};

module.exports = RemoteApps;
