// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/

var util = require('util');

/**
 * Default Constructor that creates a server error
 *
 * @param statusCode
 * @param body
 * @constructor
 */
function ServerError(statusCode, body) {
  try {
    if (statusCode === 502 || statusCode === 503 || statusCode === 504) {
      this.body = body;
      this.message = body;
    }
    else {
      this.body = body && typeof body === 'string' ? JSON.parse(body) : body;
      if (!this.body.message && body) {
        this.message = body;
      }
      else {
        this.message = this.body.message;
      }
    }
  }
  catch (error) {
    console.error(error);
    this.message = this.body;
  }
  this.statusCode = statusCode;
}

util.inherits(ServerError, Error);
/**
 * Return the stringified format of the error.
 *
 */
ServerError.prototype.toString = function() {
  var prop;
  var i;
  var e;
  if (this.body && this.body.errors) {
    for (i = 0; i < this.body.errors.length; i++) {
      e = this.body.errors[i];
      for (prop in e) {
        return e[prop];
      }
    }
  }
  var error;
  try {
    error = typeof this.body === 'string' ?
      this.body : JSON.stringify(this.body);
    return error;
  }
  catch (ex) {
    error = this.body;
  }

  if (this.message) {
    return this.message;
  }

  if (this.body && this.body.message) {
    return this.body.message;
  }
  if (!error) {
    return this.body;
  }
  if (error.message) {
    return error.message;
  }
  if (error.error) {
    if (typeof error.error === 'object') {
      var str = '';
      for (prop in error.error) {
        str += prop + ': ' + error.error[prop];
        str += ' -- ';
      }
      //str += '\n';
      return str;
    }
    else {
      for (i = 0; i < error.error.length; i++) {
        e = error.error[i];
        for (prop in e) { //jshint ignore:line
          return e[prop];
        }
      }
    }
  }

  return this.body;
};

module.exports = ServerError;


